import numpy as np
import argparse
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import pandas as pd
import os
try:
    import seaborn as sns
    cols = np.array(sns.color_palette("husl", 8))
    paired = np.array(sns.color_palette("Paired", 10)[2:])

except ImportError:
    print 'Fancy plots disabled as Seaborn is not installed'

""" 

This module should load the results from a tab-delimitered csv-file
into some data-structure, and then plot it

"""

def import_and_format(filename, separator):
    """
    Read csv file and infer from the filename the parameters
    of the network.

    Will fail if the csv is not structured as expected.

    """
    data = pd.read_csv(filename, sep=separator)
    data['n'] = np.nan
    data['mu_t'] = np.nan
    data['mu_w'] = -1
    data['transformation'] = 'A'
    
    for i in xrange(data.shape[0]):
        filename = data.loc[i, 'File']
        properties = filename.split('_')
        properties[-1] = properties[-1][:-4]
        for p in properties:
            if p.startswith('no'):
                data['transformation'][i] = str(p)
            elif p.startswith('n'):
                data['n'][i] = int(p[1:])
            elif p.startswith('mut'):
                data['mu_t'][i] = int(p[3:])
            elif p.startswith('muw'):
                data['mu_w'][i] = int(p[3:])
            else:
                data['transformation'][i] = str(p)
    return data

def plot(filepath):
    """
    Reads the output of suite.py and 
    plots the performance of the methods.
    
    """
    data = import_and_format(filepath, '\t')
    print np.unique(data.mu_t)
    data.mu_t = data.mu_t / 100
    data.mu_t[data.mu_t == 0.35] = 0.035
    data.mu_t[data.mu_t == 0.65] = 0.065
    data.mu_t[(data.mu_t == 0.2) & (data.mu_w != -1)] = 0.02
    
    data.mu_w = data.mu_w.astype(float)
    data.mu_w = data.mu_w / 100
    data.transformation[data.transformation == "pow2"] = "A^2"
    data.transformation[data.transformation == "pow3"] = "A^3"
    data.transformation[data.transformation == "exp"] = "exp(A)"
    data.method[data.method == "luv"] = "Louvain"
    data.method[data.method == "rank"] = "Rank"
    data.method[data.method == "prop"] = "Propagation"
    data.method[data.method == "dissolve"] = "Dissolve"
    means = data.groupby(['method',
                          'n',
                          'mu_t',
                          'mu_w',
                          'transformation']).mean()

    methods = ["Louvain", "Rank", "Dissolve"]
    transformations = ["A", "A^2", "A^3", "exp(A)"]    
    
    binary = means.query("mu_w == -0.01")
    binary.index = binary.index.droplevel("mu_w")

    coolness = means.query("mu_t == 0.99")

    weighted = means.query("mu_t < 0.10")
    weighted = weighted.drop("norest", level="transformation")


    
    
    for metric in ["NMI", "NVI"]:
        # Compare methods on binary networks

        for i, n in enumerate((1000, 5000)):
            fig = plt.figure(figsize = (4, 4), tight_layout=True)
            axis = fig.add_subplot(1,1,1)
            
            xs = binary.xs([n, "A"], level=['n', 'transformation'])
            xs[metric].unstack().T.plot(ax=axis, color=cols[[0, 2, 5, 6]])
            axis.set_ylim(0.0, 1.0)
            axis.set_xlim(0.2, 0.9)
            # axis.set_title("n = {}".format(n))
            
            if metric == "NVI":
                axis.set_ylabel("Normalized Variation of Information")
                axis.legend(loc="upper left")
            else:
                axis.set_ylabel("Normalized Mutual Information")
                axis.legend(loc="lower left")
            if i > 0:
                axis.legend().set_visible(False)

            fig.savefig("../Master/figures/binary_compare/binary_compare_{}_{}_xx.png".format(metric, n), dpi=600)
            plt.close(fig)
        continue
        # Plot all transformations of binary networks for all methods
        for n in (1000, 5000):
            for i, method in enumerate(methods):
                fig = plt.figure(figsize=(4, 4), tight_layout=True)
                axis = fig.add_subplot(1,1,1)
                xs = binary.xs([n, method], level=['n', 'method'])
                xs[metric].unstack().plot(ax=axis, color=cols[[3, 1, 5, 7]])
                axis.set_ylim(0.0, 1.0)
                axis.set_xlim(0.2, 0.9)
                if metric == "NVI":
                    axis.set_ylabel("Normalized Variation of Information")
                    axis.legend(loc="upper left")
                else:
                    axis.set_ylabel("Normalized Mutual Information")
                    axis.legend(loc="lower left")
                if i > 0:
                    axis.legend().set_visible(False)
                fig.savefig("../Master/figures/binary_methods/binary_methods_n{}_{}_{}".format(n, metric, method), dpi=600)
                plt.close(fig)

        # Compare methods on weighted graph
       
        for i, n in enumerate((1000, 5000)):
            for j, mu in enumerate((0.05, 0.08)):
                fig = plt.figure(figsize = (4, 4), tight_layout=True)
                axis = fig.add_subplot(1,1,1)
                xs = weighted.xs([n, mu, "A"], level=['n', 'mu_t', 'transformation'])
                xs[metric].unstack().T.drop("Propagation", 1).plot(ax=axis, color=cols[[0,2,6]])
                axis.set_ylim(0.0, 1.0)
                axis.set_xlim(0.2, 0.9)
                if metric == "NVI":
                    axis.set_ylabel("Normalized Variation of Information")
                    axis.legend(loc="upper left")
                else:
                    axis.set_ylabel("Normalized Mutual Information")
                    axis.legend(loc="lower left")
                # axis.set_title("n = {}, mu_t = {}".format(n, mu*10))

                if j != 0:
                    axis.legend().set_visible(False)
                fig.savefig("../Master/figures/weighted_compare/weighted_compare_{}_n{}_mu{}.png".format(metric, n, int(mu*100)), dpi=600)
                plt.close(fig)

        # Plot the transformations on the weighted network
        for n in ((5000,)):
            for mu in [0.02, 0.035, 0.05, 0.065, 0.08]:
                for i, method in enumerate(["Louvain"]):
                    fig = plt.figure(figsize = (4, 4), tight_layout=True)
                    axis = fig.add_subplot(1,1,1)
                    print np.unique(data.mu_t)
                    print n, mu, method, metric
                    xs = weighted.xs([method, n, mu], level=['method', 'n', 'mu_t'])
                    
                    xs_col = xs[metric].unstack()
                    print xs_col
                    xs_col.plot(ax=axis, color=cols[[3, 1, 5, 7]])

                    axis.set_ylim(0.0, 1.0)
                    axis.set_xlim(0.2, 0.9)
                    if metric == "NVI":
                        axis.set_ylabel("Normalized Variation of Information")
                        axis.legend(loc="upper left")
                    else:
                        axis.set_ylabel("Normalized Mutual Information")
                        axis.legend(loc="lower left")
                    # axis.set_title(method)
                    if i > 0:
                        axis.legend().set_visible(False)
                    fig.savefig("../Master/figures/weighted_methods/weighted_methods_n{}_mu{}_{}_{}.png".format(n, int(mu*100), metric, method), dpi=600)
                    plt.close(fig)

        # Plot the results for transformation when mu_t = mu_w
        for i, method in enumerate(methods):
            fig = plt.figure(figsize = (4, 4), tight_layout=True)
            axis = fig.add_subplot(1,1,1)
            xs = coolness.xs([5000, 0.99, method], level=['n', "mu_t", 'method'])
            xs_col = xs[metric].unstack()
            xs_col.plot(ax=axis, color=cols[[3, 1, 5, 7]])
            axis.set_ylim(0.0, 1.0)
            axis.set_xlim(0.2, 0.9)
            if metric == "NVI":
                axis.set_ylabel("Normalized Variation of Information")
                axis.legend(loc="upper left")
            else:
                axis.set_ylabel("Normalized Mutual Information")
                axis.legend(loc="lower left")
            # axis.set_title(method)
            if i > 0:
                axis.legend().set_visible(False)
            fig.savefig("../Master/figures/mutmuw/mutmuw_{}_{}.png".format(metric, method), dpi=600)
            plt.close(fig)

        # Compare methods
        fig = plt.figure(figsize = (4, 4), tight_layout=True)
        axis = fig.add_subplot(1,1,1)
        xs = coolness.xs([5000, 0.99, 'A'], level=['n', "mu_t", 'transformation'])
        xs_col = xs[metric].unstack().T
        xs_col.plot(ax=axis, color=cols[[0, 2, 5, 6]])
        axis.set_ylim(0.0, 1.0)
        axis.set_xlim(0.2, 0.9)
        if metric == "NVI":
            axis.set_ylabel("Normalized Variation of Information")
            axis.legend(loc="upper left")
        else:
            axis.set_ylabel("Normalized Mutual Information")
            axis.legend(loc="lower left")

        fig.savefig("../Master/figures/mutmuw/mutmuw_compare_{}.png".format(metric), dpi=600)
        plt.close(fig)



if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("filepath")
    args = parser.parse_args()
    if args.filepath and os.path.isfile(args.filepath):
        plot(args.filepath)
    else:
        print("Provide a valid file")
